/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: controller.cpp,v 1.26 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <vector>

#include <qslider.h>
#include <qwidget.h>
#include <qwhatsthis.h>

#include "controller.h"
#include "dvddialog.h"
#include "mainwindow.h"
#include "mixer.h"
#include "mplayer.h"
#include "opendialog.h"
#include "optionsdialog.h"
#include "playlistmodel.h"

using namespace emotion;

Controller::Controller(PlaylistModel *model, MainWindow *view)
	: _model(model),
	_view(view),
	_playingpos(-1),
	_loop(false)
{
	_mplayer = new Mplayer();

	QObject::connect(_mplayer, SIGNAL(mplayerEOF()), this,
			SLOT(mplayerEOF()));
	//QObject::connect(_mplayer, SIGNAL(mplayerKilled()), this,
	//		SLOT(mplayerKilled()));
	QObject::connect(_mplayer, SIGNAL(mplayerQuit()), this,
			SLOT(mplayerQuit()));
	QObject::connect(_mplayer, SIGNAL(mplayerOutput(const char*)), this,
			SLOT(mplayerOutput(const char*)));
	QObject::connect(_mplayer, SIGNAL(mplayerPercent(int)), this,
			SLOT(mplayerPercent(int)));
}

Controller::~Controller()
{
	delete _mplayer;
}

void Controller::displayOpenDialog(OpenDialog::Mode mode,
		bool clearlist)
{
	OpenDialog od (_view);
	od.setMode(mode);

	if (od.exec() == OpenDialog::Accepted)
	{
		if (clearlist)
			_model->clear();

		_model->add (od.selectedItems());
	}
}

void Controller::actionAdd()
{
	_view->getAdd()->flash();
	displayOpenDialog(OpenDialog::ExistingFiles, false);
}

void Controller::actionAddDirectory()
{
	displayOpenDialog(OpenDialog::DirectoryOnly, false);
}

void Controller::actionOpen()
{
	_view->getOpen()->flash();
	displayOpenDialog(OpenDialog::ExistingFiles, true);
}

void Controller::actionRemove()
{
	_view->getRemove()->flash();
	vector<int> rows (_view->getSelection());
	_model->remove(rows);
}

void Controller::actionMoveDown()
{
	_view->getPlaylistDown()->flash();
	vector<int> rows (_view->getSelection());
	
	if (rows.empty())
		return;

	// if the last item is selected, do not move
	if (rows[rows.size()-1] == _model->size()-1)
		return;

	for (vector<int>::iterator it=rows.end() - 1;
			it >= rows.begin();
			it--)
	{
		_model->swap(*it, (*it) + 1);
		(*it)++;
	}

	_view->setSelection (rows);
}

void Controller::actionConfiguration()
{
	_view->getConfiguration()->flash();
	OptionsDialog dialog(_view);
	dialog.exec();
}

void Controller::actionMoveUp()
{
	_view->getPlaylistUp()->flash();
	vector<int> rows (_view->getSelection());

	if (rows.empty())
		return;

	if (rows[0] == 0)
		return;

	for (vector<int>::iterator it=rows.begin();
			it != rows.end();
			it++)
	{
		_model->swap(*it, (*it) - 1);
		(*it)--;
	}

	_view->setSelection(rows);
}

void Controller::actionSkipForward()
{
	_view->getSkipForward()->flash();
	// stop playing current item
	_mplayer->quit ();

	_model->advance();

	// if we were playing before we switched, play this one too
	if (_playingpos >= 0 && _model->getPosition() >= 0)
		actionPlay();
	else if (_playingpos >= 0 && _loop)
		// if we were playing and are in loop mode
	{
		if (_model->getPosition() == -1 && (!_model->empty()))
			// if we just passed the end of the list
		{
			// then loop around
			_model->setPosition(0);
			actionPlay();
		}
	}
	else
		actionStop();
}

void Controller::actionSkipBack()
{
	_view->getSkipBack()->flash();
	// stop playing current item
	_mplayer->quit ();

	_model->retreat();

	// if we were playing before we switched, play this one too
	if (_playingpos > 0 && _model->getPosition() >= 0)
		actionPlay();
	else
		actionStop();
}

void Controller::actionPlay()
{
	_view->getPlay()->flash();
	int pos = _model->getPosition();

	if (_model->empty())
		// If there is nothing in the playlist, display open dialog and play if
		// something was selected
	{
		actionOpen();
		if (!_model->empty())
			actionPlay();
		return;
	}
	else if (pos == -1)
	{
		actionSkipForward();
		actionPlay();
		return;
	}

	MplayerFile *file = _model->get(pos);

	_mplayer->play(*file);

	_playingpos = pos;

	_view->enablePosition();
}

void Controller::actionStop()
{
	_view->getStop()->flash();
	_view->disablePosition();
	_playingpos = -1;
	_mplayer->quit();
}

void Controller::actionPlay(int row)
{
	if (row >= _model->size())
		return;
	_view->getPlay()->flash();
	_model->setPosition(row);
	actionPlay();
}

void Controller::mplayerEOF()
{
	actionSkipForward();
}

void Controller::mplayerKilled()
{
	actionSkipForward();
}

void Controller::mplayerQuit()
{
	actionStop();
}

void Controller::fileRemoved(const vector<int> &files)
{
	if (_playingpos == -1)
		return;

	bool restart = false;

	for (vector<int>::const_iterator it=files.begin();
			it != files.end();
			it++)
	{
		if (*it == _playingpos)
		{
			actionStop();
			restart = true;
		}
	}

	if (restart && !_model->empty())
		actionPlay();
}

void Controller::actionRewind()
{
	_view->getRewind()->flash();
	_mplayer->back();
}

void Controller::actionForward()
{
	_view->getForward()->flash();
	_mplayer->forward();
}

void Controller::actionPause()
{
	_view->getPause()->flash();
	_mplayer->pause();
}

void Controller::actionFullscreen()
{
	_view->getFullscreen()->flash();
	_mplayer->toggleFullscreen();
}

void Controller::actionMute()
{
	_view->getMute()->flash();
	Mixer *m = Mixer::instance();

	m->toggleMute();

	if (m->isMuted())
		_view->getVolumeSlider()->setDisabled(TRUE);
	else
		_view->getVolumeSlider()->setDisabled(FALSE);
}

void Controller::actionToggleSubs()
{
	_view->getSubs()->flash();
	_mplayer->toggleSubtitles();
	_view->getSubs()->setOn(_mplayer->getSubs());
}

void Controller::actionLoop(bool loop)
{
	_loop = loop;

	emit loopToggled(loop);
}

void Controller::actionLoop()
{
	actionLoop(!_loop);
}

bool Controller::getLoop() const
{
	return _loop;
}

void Controller::actionClear()
{
	_model->clear();
}

void Controller::actionHelp()
{
	_view->getHelp()->flash();
	QWhatsThis::enterWhatsThisMode();
}

void Controller::actionDVD()
{
	_view->getDVD()->flash();

	DVDDialog dd(_view);

	if (dd.exec() == DVDDialog::Accepted)
	{
		bool playing = (_playingpos != -1);

		_model->clear();

		QString filename;
		filename.sprintf("dvd://%d", dd.getTitle());
		MplayerFile file(filename, PlayOptions());
		file.setName(filename);

		vector<MplayerFile> files;
		files.push_back(file);
		_model->add(files);
		
		if (playing)
			actionPlay();
	}
}

void Controller::mplayerOutput(const char *str)
{
	_view->modelOutput(str);
}

void Controller::actionConsole()
{
	_view->showConsole();
}

void Controller::actionSeek()
{
	_mplayer->seekTo(_view->getSeekSlider()->value());
}

void Controller::actionSeek(int pos)
{
	_mplayer->seekTo(pos);
}

void Controller::mplayerPercent(int pos)
{
	_view->displayPosition (pos);
}
