/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: emotionapp.cpp,v 1.3 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <iostream>

#include <qaction.h>
#include <qapplication.h>
#include <qdial.h>
#include <qmainwindow.h>
#include <qnamespace.h>
#include <qobject.h>
#include <qslider.h>

#include "emotionapp.h"
#include "controller.h"
#include "globalconfiguration.h"
#include "mainwindow.h"
#include "mixer.h"
#include "playlistmodel.h"

using namespace emotion;

emotionApp::emotionApp(PlaylistModel *model, QMainWindow *view, Controller
		*controller)
	: _model(model),
	_controller (controller),
	_view (view)
{
	/* Restore geometry */
	GlobalConfiguration *conf = GlobalConfiguration::instance();
	QPoint position (QPoint(conf->getInt("geometry_xpos"),
				conf->getInt("geometry_ypos")));
	QSize size (QSize (conf->getInt("geometry_xsize"),
				conf->getInt("geometry_ysize")));

	_view->resize (size);
	_view->move (position);

	QString filename;
	filename = getenv("HOME");
	filename += "/.emotionplaylist";
	if (QFile(filename).exists())
	{
		_model->read(filename);
		_model->setPosition(conf->getInt("position"));
	}
}

emotionApp::~emotionApp()
{
	// save geometry
	QPoint position(_view->pos());
	QSize size(_view->size());
	GlobalConfiguration *conf = GlobalConfiguration::instance();
	conf->setInt ("geometry_xpos", position.x());
	conf->setInt ("geometry_ypos", position.y());
	conf->setInt ("geometry_xsize", size.width());
	conf->setInt ("geometry_ysize", size.height());

	conf->setBool ("loop", _controller->getLoop());

	// save playlist
	QString filename;
	filename = getenv("HOME");
	filename += "/.emotionplaylist";
	_model->write(filename);
	conf->setInt("position", _model->getPosition());

	delete _view;
	delete _controller;
	delete _model;
}

emotionApp *emotionApp::create(int argc, char **argv)
{
	MainWindow *view = new MainWindow (0, QObject::tr("eMotion"));
	PlaylistModel *model = new PlaylistModel ();
	Controller *controller = new Controller (model, view);

	//////////////////////////////////////////////////////////
	// Build view

	// Toolbar - Quit button
	QObject::connect(view->getQuit(), SIGNAL(clicked()),
			qApp, SLOT(quit()));

	// Toolbar - Main menu
	QPopupMenu *mainmenu = view->getMenu();
	// Main menu
	mainmenu->insertItem ("&About eMotion", view, SLOT(about()));
	mainmenu->insertSeparator ();
	mainmenu->insertItem ("&Open", controller, SLOT(actionOpen()));
	mainmenu->insertItem ("&Configuration", controller, SLOT(actionConfiguration()));
	mainmenu->insertSeparator ();
	mainmenu->insertItem ("&Mplayer Output", controller, SLOT(actionConsole()));
	mainmenu->insertSeparator ();
	mainmenu->insertItem ("&Quit", qApp, SLOT(quit()));

	// Toolbar - Add File Menu
	QPopupMenu *addmenu = view->getAddMenu();
	// Add file menu
	addmenu->insertItem ("Add &File(s)", controller, SLOT(actionAdd()));
	addmenu->insertItem ("Add &Directory Recursively", controller, SLOT(actionAddDirectory()));

	// Toolbar - Remove File Menu
	QPopupMenu *rmmenu = view->getRemoveMenu();
	// Add file menu
	rmmenu->insertItem ("Remove &File(s)", controller, SLOT(actionRemove()));
	rmmenu->insertItem ("Remove &All", controller, SLOT(actionClear()));

	//////////////////////////////////////////////////////////
	// Connect model to view
	QObject::connect(model, SIGNAL(fileAdded(const vector<QString>&, int)), view,
			SLOT(modelFileAdded(const vector<QString>&, int)));
	QObject::connect(model, SIGNAL(fileDeselected(int)), view,
			SLOT(modelFileDeselected(int)));
	QObject::connect(model, SIGNAL(fileRemoved(const vector<int>&)), view,
			SLOT(modelFileRemoved(const vector<int>&)));
	QObject::connect(model, SIGNAL(fileSwapped(int,int)), view,
			SLOT(modelFileSwapped(int,int)));
	QObject::connect(model, SIGNAL(fileSelected(int)), view,
			SLOT(modelFileSelected(int)));

	//////////////////////////////////////////////////////////
	// Connect view to controller
	QObject::connect(view->getAdd(), SIGNAL(clicked()),
			controller, SLOT(actionAdd()));
	QObject::connect(view->getConfiguration(), SIGNAL(clicked()),
			controller, SLOT(actionConfiguration()));
	QObject::connect(view->getDVD(), SIGNAL(clicked()),
			controller, SLOT(actionDVD()));
	QObject::connect(view->getForward(), SIGNAL(clicked()),
			controller, SLOT(actionForward()));
	QObject::connect(view->getFullscreen(), SIGNAL(clicked()),
			controller, SLOT(actionFullscreen()));
	QObject::connect(view->getHelp(), SIGNAL(clicked()),
			controller, SLOT(actionHelp()));
	QObject::connect(view->getLoop(), SIGNAL(toggled(bool)),
			controller, SLOT(actionLoop(bool)));
	QObject::connect(view->getMute(), SIGNAL(clicked()),
			controller, SLOT(actionMute()));
	QObject::connect(view->getOpen(), SIGNAL(clicked()),
			controller, SLOT(actionOpen()));
	QObject::connect(view->getPause(), SIGNAL(clicked()),
			controller, SLOT(actionPause()));
	QObject::connect(view->getPlay(), SIGNAL(clicked()),
			controller, SLOT(actionPlay()));
	QObject::connect(view->getPlaylistDown(), SIGNAL(clicked()),
			controller, SLOT(actionMoveDown()));
	QObject::connect(view->getPlaylistUp(), SIGNAL(clicked()),
			controller, SLOT(actionMoveUp()));
	QObject::connect(view->getRemove(), SIGNAL(clicked()),
			controller, SLOT(actionRemove()));
	QObject::connect(view->getRewind(), SIGNAL(clicked()),
			controller, SLOT(actionRewind()));
	QObject::connect(view->getSkipBack(), SIGNAL(clicked()),
			controller, SLOT(actionSkipBack()));
	QObject::connect(view->getSkipForward(), SIGNAL(clicked()),
			controller, SLOT(actionSkipForward()));
	QObject::connect(view->getStop(), SIGNAL(clicked()),
			controller, SLOT(actionStop()));
	QObject::connect(view->getSubs(), SIGNAL(clicked()),
			controller, SLOT(actionToggleSubs()));

	QObject::connect(view->getVolumeSlider(), SIGNAL(valueChanged(int)),
			Mixer::instance(), SLOT(setVolume(int)));

	QObject::connect(view->getSeekSlider(), SIGNAL(sliderReleased()),
			controller, SLOT(actionSeek()));

	QObject::connect(view, SIGNAL(fileClicked(int)),
			controller, SLOT(actionPlay(int)));

	//////////////////////////////////////////////////////////
	// Misc. connections
	QObject::connect(Mixer::instance(), SIGNAL(muted(bool)),
			view->getMute(), SLOT(setOn(bool)));
	QObject::connect(Mixer::instance(), SIGNAL(volumeChanged(int)),
			view, SLOT(volumeChanged(int)));
	QObject::connect(controller, SIGNAL(loopToggled(bool)),
			view->getLoop(), SLOT(setOn(bool)));

	QObject::connect(model, SIGNAL(fileRemoved(const vector<int>&)),
			controller, SLOT(fileRemoved(const vector<int>&)));

	controller->actionLoop(GlobalConfiguration::instance()->getBool("loop"));

	//////////////////////////////////////////////////////////
	// Instantiate application
	emotionApp *app = new emotionApp(model, view, controller);

	//////////////////////////////////////////////////////////
	// Keyboard shortcuts
	app->_actadd = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_A, view);
	app->_actconfiguration = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_P, view);
	app->_actdvd = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_D, view);
	app->_actforward = new QAction (QString::null, QString::null,
			Qt::Key_Right, view);
	app->_actfullscreen = new QAction (QString::null, QString::null,
			Qt::Key_F, view);
	app->_acthelp = new QAction (QString::null, QString::null,
			Qt::Key_F1, view);
	app->_actloop = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_L, view);
	app->_actmovedown = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_Down, view);
	app->_actmoveup = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_Up, view);
	app->_actmute = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_M, view);
	app->_actopen = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_O, view);
	app->_actpause = new QAction (QString::null, QString::null,
			Qt::Key_Space, view);
	app->_actplay = new QAction (QString::null, QString::null,
			Qt::Key_P, view);
	app->_actquit = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_Q, view);
	app->_actremove = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::SHIFT+Qt::Key_A, view);
	app->_actrewind = new QAction (QString::null, QString::null,
			Qt::Key_Left, view);
	app->_actsubs = new QAction (QString::null, QString::null,
			Qt::CTRL+Qt::Key_S, view);
	app->_actskipback = new QAction (QString::null, QString::null,
			Qt::Key_Prior, view);
	app->_actskipforward = new QAction (QString::null, QString::null,
			Qt::Key_Next, view);
	app->_actstop = new QAction (QString::null, QString::null,
			Qt::Key_S, view);
	app->_actvolumedown = new QAction (QString::null, QString::null,
			Qt::Key_Down, view);
	app->_actvolumeup = new QAction (QString::null, QString::null, Qt::Key_Up,
			view);

	QObject::connect(app->_actadd, SIGNAL(activated()),
			controller, SLOT(actionAdd()));
	QObject::connect(app->_actconfiguration, SIGNAL(activated()),
			controller, SLOT(actionConfiguration()));
	QObject::connect(app->_actdvd, SIGNAL(activated()),
			controller, SLOT(actionDVD()));
	QObject::connect(app->_actforward, SIGNAL(activated()),
			controller, SLOT(actionForward()));
	QObject::connect(app->_actfullscreen, SIGNAL(activated()),
			controller, SLOT(actionFullscreen()));
	QObject::connect(app->_acthelp, SIGNAL(activated()),
			controller, SLOT(actionHelp()));
	QObject::connect(app->_actloop, SIGNAL(activated()),
			controller, SLOT(actionLoop()));
	QObject::connect(app->_actmovedown, SIGNAL(activated()),
			controller, SLOT(actionMoveDown()));
	QObject::connect(app->_actmoveup, SIGNAL(activated()),
			controller, SLOT(actionMoveUp()));
	QObject::connect(app->_actmute, SIGNAL(activated()),
			controller, SLOT(actionMute()));
	QObject::connect(app->_actopen, SIGNAL(activated()),
			controller, SLOT(actionOpen()));
	QObject::connect(app->_actpause, SIGNAL(activated()),
			controller, SLOT(actionPause()));
	QObject::connect(app->_actplay, SIGNAL(activated()),
			controller, SLOT(actionPlay()));
	QObject::connect(app->_actquit, SIGNAL(activated()),
			qApp, SLOT(quit()));
	QObject::connect(app->_actremove, SIGNAL(activated()),
			controller, SLOT(actionRemove()));
	QObject::connect(app->_actrewind, SIGNAL(activated()),
			controller, SLOT(actionRewind()));
	QObject::connect(app->_actsubs, SIGNAL(activated()),
			controller, SLOT(actionToggleSubs()));
	QObject::connect(app->_actskipback, SIGNAL(activated()),
			controller, SLOT(actionSkipBack()));
	QObject::connect(app->_actskipforward, SIGNAL(activated()),
			controller, SLOT(actionSkipForward()));
	QObject::connect(app->_actstop, SIGNAL(activated()),
			controller, SLOT(actionStop()));
	QObject::connect(app->_actvolumedown, SIGNAL(activated()),
			Mixer::instance(), SLOT(decVolume()));
	QObject::connect(app->_actvolumeup, SIGNAL(activated()),
			Mixer::instance(), SLOT(incVolume()));

	//////////////////////////////////////////////////////////
	// Command line arguments
	vector<MplayerFile> args;
	for (int i=1; i<argc; i++)
		args.push_back(MplayerFile(argv[i], PlayOptions()));
	if (args.size() > 0)
	{
		model->clear();
		model->add(args);
	}

	return app;
}

QWidget *emotionApp::getView()
{
	return _view;
}

void emotionApp::show()
{
	_view->show();
}
