/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: imagebutton.cpp,v 1.11 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <iostream>

#include <qpainter.h>
#include <qpixmap.h>
#include <qpopupmenu.h>

#include "imagebutton.h"

using namespace emotion;

ImageButton::ImageButton (QWidget *parent, const IconSet &iconset)
	: QButton (parent),
	_icons (iconset),
	_popup (NULL),
	_delay (0),
	_menutimer (this),
	_menudisplayed (false),
	_autoraise (true),
	_buttonstate (0),
	_togglestate (NORMAL)
{
	init ();
}

ImageButton::~ImageButton ()
{
}

void ImageButton::init ()
{
	setMinimumSize(_icons.getPixmap(IconSet::NORMAL).size());

	QObject::connect (this, SIGNAL(pressed()), this, SLOT(pressed()));
	QObject::connect (this, SIGNAL(released()), this, SLOT(released()));

	QObject::connect (&_menutimer, SIGNAL(timeout()), this, SLOT(displayMenu()));

	QObject::connect(this, SIGNAL(toggled(bool)), this, SLOT(toggledStateChanged(bool)));
}

void ImageButton::toggledStateChanged(bool state)
{
	if (state)
		_togglestate = TOGGLED;
	else
		_togglestate = NORMAL;
}

void ImageButton::drawButton (QPainter *painter)
{
	int togglestate;
	QPixmap icon;

	if (_togglestate == TOGGLED)
		togglestate = IconSet::ON;
	else
		togglestate = IconSet::OFF;

	if (_buttonstate & DOWN)
		icon = _icons.getPixmap(IconSet::DOWN,
					togglestate);
	else if (_autoraise && (_buttonstate & MOUSEOVER))
		icon = _icons.getPixmap(IconSet::ACTIVE,
					togglestate);
	else
		icon = _icons.getPixmap(IconSet::NORMAL,
					togglestate);

	// draw image centered vertically
	int yoff = (height() / 2) - (icon.height()/2);
	painter->drawPixmap (0, yoff, icon);
}

void ImageButton::setToggleButton(bool toggleness)
{
	QButton::setToggleButton(toggleness);
}

void ImageButton::displayMenu ()
{
	if (isDown())
		popup ();
	_menutimer.stop ();
}

void ImageButton::pressed ()
{
	_buttonstate |= DOWN;
	repaint();
	_menutimer.start(_delay);
}

void ImageButton::setPopupDelay (int millis)
{
	_delay = millis;
}

void ImageButton::released ()
{
	_buttonstate &= ~DOWN;
	repaint();
}

void ImageButton::enterEvent (QEvent *ev)
{
	_buttonstate |= MOUSEOVER;
	repaint();
}

void ImageButton::leaveEvent (QEvent *ev)
{
	_buttonstate &= ~MOUSEOVER;
	repaint();
}

void ImageButton::aboutToHideMenu ()
{
	_menudisplayed = false;
	_buttonstate &= ~DOWN;
	repaint();
}

void ImageButton::setPopup (QPopupMenu *popup)
{
	_popup = popup;
	if (_popup)
	{
		QObject::connect (_popup, SIGNAL(aboutToShow()), this, SLOT(aboutToShowMenu()));
		QObject::connect (_popup, SIGNAL(aboutToHide()), this, SLOT(aboutToHideMenu()));
	}
}

void ImageButton::aboutToShowMenu ()
{
	_menudisplayed = true;
	_buttonstate |= DOWN;
	repaint();
}

void ImageButton::popup ()
{
	if (_popup)
	{
		QPoint pos (mapToGlobal (QPoint(0,
						_icons.getPixmap(IconSet::NORMAL).height() - 4)));
		_popup->popup (pos);
	}
}

bool ImageButton::autoRaise () const
{
	return _autoraise;
}

void ImageButton::setAutoRaise (const bool autoraise)
{
	_autoraise = autoraise;
}

void ImageButton::flash ()
{
	_buttonstate |= DOWN;
	repaint();

	QTimer::singleShot(200, this, SLOT(released()));
}

void ImageButton::setOn(bool on)
{
	if (on)
		_togglestate = TOGGLED;
	else
		_togglestate = NORMAL;

	repaint();
	QButton::setOn(on);
}
