/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: mainwindow.h,v 1.37 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#ifndef __MAINWINDOW_H
#define __MAINWINDOW_H

#include <vector>

#include <qmainwindow.h>

#include "errormediator.h"
#include "exception.h"
#include "imagebutton.h"
#include "playlist.h"

using namespace std;

class QErrorMessage;
class QHBox;
class QLabel;
class QPopupMenu;
class QSlider;
class QToolBar;
class QVBox;

namespace emotion
{
	class ConsoleDialog;
	class ImageButton;
	class Playlist;

	/** The view part of the MVC trio */
	class MainWindow
		: public QMainWindow, Acceptor
	{
			Q_OBJECT
		public:
			MainWindow (QWidget *parent = 0, const char *name = 0);
			~MainWindow ();

			/** See the ErrorMediator and Acceptor classes. This is called when an error occurs. */
			virtual void acceptError (const Exception &e);

			virtual ImageButton *getAdd();
			virtual QPopupMenu  *getAddMenu();
			virtual ImageButton *getConfiguration();
			virtual ImageButton *getDVD();
			virtual ImageButton *getForward();
			virtual ImageButton *getFullscreen();
			virtual ImageButton *getHelp();
			virtual ImageButton *getLoop();
			virtual QPopupMenu  *getMenu();
			virtual ImageButton *getMute();
			virtual ImageButton *getOpen();
			virtual ImageButton *getPause();
			virtual ImageButton *getPlay();
			virtual ImageButton *getPlaylistDown();
			virtual ImageButton *getPlaylistUp();
			virtual ImageButton *getQuit();
			virtual ImageButton *getRemove();
			virtual QPopupMenu  *getRemoveMenu();
			virtual ImageButton *getRewind();
			virtual QSlider     *getSeekSlider();
			virtual ImageButton *getSkipBack();
			virtual ImageButton *getSkipForward();
			virtual ImageButton *getSubs();
			virtual ImageButton *getStop();
			virtual QSlider     *getVolumeSlider();

			/** Returns a vector containing the indexes of every selected item
			 * in the playlist */
			virtual vector<int> getSelection() const;
			/** Select the indexes contained in the vector provided in the
			 * playlist */
			virtual void setSelection(const vector<int> &);

		public slots:
			/** Display this window (this method is extended a bit from its
			 * parent class to do some other work */
			virtual void show ();

			/** Display the about dialog */
			virtual void about ();

			/** Called when the playlist contents have changed and the view
			 * needs to be updated */
			virtual void modelFileAdded(const vector<QString> &, int);
			/** Called when the playlist contents have changed and the view
			 * needs to be updated */
			virtual void modelFileRemoved(const vector<int> &);
			/** Called when the playlist contents have changed and the view
			 * needs to be updated */
			virtual void modelFileSwapped(int, int);
			/** Called when the playlist's current position becomes deselected
			 * (i.e. when advancing in the playlist). This is seperate from the
			 * Seperated slot because if you are at the end of the playlist, a
			 * file can be deselected without another being selected. */
			virtual void modelFileDeselected(int);
			virtual void modelFileSelected(int);

			/** Handle mplayer's printed output */
			virtual void modelOutput(const QString&);

			/** Update the volume slider when the volume is changed */
			virtual void volumeChanged (int);

			/** Display the mplayer output dialog */
			virtual void showConsole();

			/** Update the seek bar */
			virtual void displayPosition (int percent);

			virtual void disablePosition();
			virtual void enablePosition();

		signals:
			/** Called when a file is double-clicked in the playlist.
			 * Internally, this signal is just forwarded from the Playlist
			 * widget */
			virtual void fileClicked(int);

			virtual void seekingBegun();
			virtual void seekingFinished();

		private slots:
			/** Slot that just forwards the fileClicked(int) signal. See above
			 * */
			virtual void listFileClicked(int);

		private:
			/** The toolbar at the top, with the emotion logo/menu, open, quit,
			 * etc. */
			QToolBar *_toolbar;

			/** Toplevel layout widget aligns things vertically */
			QVBox *_vlayout;
			/** Horizontal box contains a toolbar with the play controls and
			 * such in it */
			QHBox *_buttonbar;

			/** Playlist widget takes up most of the rest of the screen */
			Playlist *_playlist;

			ImageButton *_menubutton;
			QPopupMenu *_menu;
			ImageButton *_open;
			ImageButton *_dvd;
			ImageButton *_options;
			ImageButton *_help;
			ImageButton *_quit;
			QLabel *_toolbarspacer;

			/** Controls toolbar is the widget that actually contains the play
			 * controls */
			QToolBar *_controls;
			ImageButton *_skipback;
			ImageButton *_back;
			ImageButton *_stop;
			ImageButton *_pause;
			ImageButton *_play;
			ImageButton *_forward;
			ImageButton *_skipforward;
			/** Spacer here, here just to get some widgets to the right side of
			 * the screen */
			QLabel *_controlspacer;
			ImageButton *_subs;
			ImageButton *_fullscreen;
			ImageButton *_mute;
			QSlider *_volume;

			QLabel *_seekspacer1;
			QLabel *_seekspacer2;
			QSlider *_seek;

			ConsoleDialog *_console;

			QErrorMessage *_errordlg;
	};
}

#endif
