/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: mixer.cpp,v 1.13 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include <math.h>
#include <sys/types.h>

#include "globalconfiguration.h"
#include "errormediator.h"
#include "exception.h"
#include "mixer.h"

using namespace emotion;

auto_ptr<Mixer> Mixer::_instance;

Mixer::Mixer(bool log)
	: _lastvolume (0),
	_log (log),
	_step (10),
	_muted (false),
	_oldvol(0),
	_timer (this)
{
	QString mixername = GlobalConfiguration::instance()->getString("mixer");
	_mixerfd = open(mixername.ascii(), O_RDWR);

	if (_mixerfd < 0)
	{
		QString message = "Unable to open device: ";
		message += mixername;
		ErrorMediator::instance()->submitError(
				Exception (message));
		return;
	}

	QObject::connect (&_timer, SIGNAL(timeout()), this, SLOT(checkVolume ()));
	_timer.start (250);
}

Mixer::~Mixer ()
{
	_timer.stop ();

	if (_muted)
		toggleMute ();

	// close (_mixerfd);
}

Mixer *Mixer::instance ()
{
	if (Mixer::_instance.get() == NULL)
		_instance.reset (new Mixer());
	return _instance.get();
}

void Mixer::toggleMute ()
{
	int res;

	if (_muted)
	{
		setVolume (_oldvol);
		_muted = false;
	}
	else
	{
		_oldvol = getVolume();
		setVolume (0);
		_muted = true;
	}

	emit muted(_muted);
}

int Mixer::getVolume () const
{
	int vol = 0;
	int res = -1;
	
	static int errors = 0;
	
	if (errors > 3)
	{
		const_cast<QTimer*>(&_timer)->stop ();
		ErrorMediator::instance()->submitError(
				Exception ("Multiple significant errors have occured while "
					"trying to read your mixer device."));
		return 0;
	}

	res = ioctl (_mixerfd, MIXER_READ(SOUND_MIXER_PCM), &vol);

	if (res == -1)
	{
		errors++;
		ErrorMediator::instance()->submitError(
			Exception ("An error occured trying to read your mixer device."));
		return 0;
	}

	return vol & 0x7f;
}

void Mixer::setVolume (int v)
{
	int vol = 0;
	int res = -1;
	
	static int errors = 0;
	
	if (errors > 3)
	{
		ErrorMediator::instance()->submitError(
			Exception ("Multiple significant errors have occured while trying "
				"to write your mixer device."));
		return;
	}

	if (v <= 0)
		v = 1;

	if (_log)
		v = (int)floor((log10(static_cast<double>(v))*50.0+0.5));

	vol = ((v << 8) & 0x7f00) | (v & 0x007f);

	res = ioctl (_mixerfd, MIXER_WRITE(SOUND_MIXER_PCM), &vol);

	if (res == -1)
	{
		errors++;
		ErrorMediator::instance()->submitError(
			Exception ("An error occured trying to write your mixer device."));
		return;
	}
}

void Mixer::incVolume ()
{
	setVolume (getVolume() + _step);
}

void Mixer::decVolume ()
{
	setVolume (getVolume() - _step);
}

int Mixer::getStep () const
{
	return _step;
}

void Mixer::setStep (int s)
{
	_step = s;
}

void Mixer::checkVolume ()
{
	int vol = getVolume ();
	if (_lastvolume != vol)
	{
		_lastvolume = vol;
		emit volumeChanged (vol);
	}
}

bool Mixer::isMuted() const
{
	return _muted;
}
