/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: mplayer.cpp,v 1.43 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include <iostream>

#include <qtimer.h>

#include "globalconfiguration.h"
#include "errormediator.h"
#include "mplayer.h"
#include "mixer.h"
#include "sighandler.h"

using namespace emotion;

Mplayer::Mplayer ()
	: _state (NOTRUNNING),
	_paused (0),
	_timer (this),
	_buffer(""),
	_quitstate(QUIT_NULL)
{
	QObject::connect (this, SIGNAL(mplayerEOF()), this, SLOT(cleanup ()));
	QObject::connect (this, SIGNAL(mplayerQuit()), this, SLOT(cleanup ()));
	QObject::connect (this, SIGNAL(mplayerKilled()), this, SLOT(cleanup ()));

	QObject::connect (&_timer, SIGNAL(timeout()), this, SLOT(pollMplayer ()));

	GlobalConfiguration *conf = GlobalConfiguration::instance();
	_fullscreen = conf->getBool("fullscreen");
	_subtitles = conf->getBool("subtitles");
}

Mplayer::~Mplayer ()
{
	quit ();

	GlobalConfiguration *conf = GlobalConfiguration::instance();
}

void Mplayer::play (const MplayerFile file)
{
	if (_state == RUNNING)
		quit ();

	if (socketpair (AF_UNIX, SOCK_STREAM, 0, _fd_send) ||
		socketpair (AF_UNIX, SOCK_STREAM, 0, _fd_recv))
	{
		ErrorMediator::instance ()->submitError (Exception(strerror(errno)));
		return;
	}

	signal (SIGPIPE, SIG_IGN);
	signal (SIGCHLD, SIG_IGN);

	register_handler (SIGTERM, this);
	register_handler (SIGSEGV, this);

	_id = fork ();

	if (!_id)
	{
		GlobalConfiguration *conf = GlobalConfiguration::instance ();

		const char *args[32];
		int i=0;

		args[i++] = conf->getString("mplayer").ascii();
		args[i++] = "-slave";
		args[i++] = "-quiet";

		QString display(conf->getString("display"));
		if (display.length())
		{
			args[i++] = "-display";
			args[i++] = display;
		}

		QString audio(conf->getString("audioout"));
		if (audio.length())
		{
			args[i++] = "-ao";
			args[i++] = audio;
		}

		if (conf->getBool("fullscreen"))
			args[i++] = "-fs";

		vector<QString> opts(file.getOptions().getOptionStrings());
		for (vector<QString>::const_iterator it = opts.begin();
				it != opts.end();
				it++)
		{
			args[i++] = it->ascii();
		}

		args[i++] = "-af";
		QString volboost;
		volboost = volboost.sprintf("volume=%d", file.getOptions().getVolumeBoost());
		args[i++] = volboost.ascii();

		args[i++] = file.getPath().ascii();
		args[i++] = NULL;

		dup2 (_fd_send[0], STDIN_FILENO);
		close (_fd_send[0]);
		close (_fd_send[1]);
		dup2 (_fd_recv[0], STDOUT_FILENO);
		dup2 (_fd_recv[0], STDERR_FILENO);
		close (_fd_send[0]);
		close (_fd_send[1]);

		execvp (args[0], (char *const *)args);
		ErrorMediator::instance ()->submitError (Exception(strerror(errno)));
		exit(-1);
	}

	close (_fd_send[0]);
	close (_fd_recv[0]);

	_notifier = new QSocketNotifier (_fd_recv[1], QSocketNotifier::Read);
	QObject::connect(_notifier, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));

	_state = RUNNING;

	_timer.start (500);
	emit mplayerStarted ();
}

void Mplayer::sendCommand (const QString &c)
{
	if (!isRunning())
		return;

	QString command(c);
	command += "\n";

	write (_fd_send[1], command.ascii(), command.length() * sizeof(char));

	if (_paused)
		_paused = 0;
}

void Mplayer::sendQuit ()
{
	sendCommand ("quit\n");
}

void Mplayer::sendTerm ()
{
	if (_state == NOTRUNNING)
		return;

	kill (_id, SIGTERM);
}

void Mplayer::sendKill ()
{
	if (_state == NOTRUNNING)
		return;

	kill (_id, SIGKILL);
}

void Mplayer::quit ()
{
	if (_state == NOTRUNNING)
		return;

	sendQuit ();

	if (pingChild ())
	{
		sleep(1);
		if (pingChild())
			sendTerm ();
	}

	if (pingChild ())
	{
		sleep(1);
		if (pingChild())
			sendKill ();
	}

	if (pingChild())
	{
		sleep(1);
		if (pingChild())
			ErrorMediator::instance ()->submitError (Exception("MPlayer is still "
						"running, and I grok not why. Sorry, I tried my best!"));
	}

	emit mplayerKilled();
}

void Mplayer::cleanup ()
{
	delete _notifier;

	close (_fd_send[1]);
	close (_fd_recv[1]);

	_timer.stop();

	_state = NOTRUNNING;

	_quitstate = QUIT_NULL;
}

void Mplayer::dataReceived (int s)
{
	int count=0;
	const int bufsize (4096);
	char buffer[bufsize];

	int avail;
	if (s < (bufsize-1))
		avail = s;
	else
		avail = bufsize-1;

	count = read (_fd_recv[1], buffer, avail);

	buffer[count] = '\0';

	if (_buffer.endsWith("\n"))
		_buffer = "";
	_buffer += buffer;
	int newline = _buffer.find('\n');
	if (newline != -1)
	{
		QString line(_buffer.left(newline+1));

		if (_buffer.startsWith("ANS"))
		{
			if (_buffer.startsWith("ANS_PERCENT_POSITION="))
			{
				int percent = atoi(_buffer.section('=', 1, 1));
				emit mplayerPercent(percent);
			}
			else if (_buffer.startsWith("ANS_VO_FULLSCREEN"))
			{
				_fullscreen = atoi(_buffer.section('=', 1, 1));
				GlobalConfiguration::instance()->setBool("fullscreen", _fullscreen);
			}
			else if (_buffer.startsWith("ANS_SUB_VISIBILITY"))
			{
				_subtitles = atoi(_buffer.section('=', 1, 1));
				GlobalConfiguration::instance()->setBool("subtitles", _subtitles);
			}
		}
		else if (_buffer.startsWith("ACT"))
		{
			if (_buffer.startsWith("ACT_QUIT_USER"))
			{
				_quitstate = QUIT_USER;
			}
			else if (_buffer.startsWith("ACT_QUIT_EOF"))
			{
				_quitstate = QUIT_EOF;
			}
		}

		_buffer = _buffer.right(newline);
	}

	emit mplayerOutput(buffer);
}

void Mplayer::pause ()
{
	if (!isRunning())
		return;

	sendCommand ("pause\n");

	_paused = ! _paused;
}

bool Mplayer::isRunning () const
{
	return (_state == RUNNING);
}

void Mplayer::forward ()
{
	if (!isRunning())
		return;

	sendCommand ("seek 5 0");
}

void Mplayer::back ()
{
	if (!isRunning())
		return;

	sendCommand ("seek -5 0");
}

bool Mplayer::pingChild ()
{
	if (!isRunning())
		return false;

	int rv = kill(_id, 0);
	
	if (rv == -1 && errno == ESRCH)
	{
		return false;
	}
	else if (rv == -1)
		perror ("pinging mplayer process");

	return true;
}

void Mplayer::pollMplayer ()
{
	if (!pingChild ())
	{
		emit mplayerPercent (0);
		if (_quitstate == QUIT_EOF)
			emit mplayerEOF ();
		else
			emit mplayerQuit ();
	}
	else if (!_paused)
		sendCommand("get_percent_pos");
}

void Mplayer::handleSignal (int sig)
{
	quit ();
	exit (- sig);
}

void Mplayer::toggleFullscreen ()
{
	sendCommand ("vo_fullscreen");
	sendCommand ("get_vo_fullscreen");
}

void Mplayer::toggleSubtitles()
{
	sendCommand ("sub_visibility");
	sendCommand ("get_sub_visibility");
}

void Mplayer::seekTo(int percent)
{
	QString cmd;
	sendCommand(cmd.sprintf("seek %d 1", percent));
}

bool Mplayer::getSubs() const
{
	return _subtitles;
}
