/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: mplayer.h,v 1.23 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#ifndef __MPLAYER_H
#define __MPLAYER_H

#include <stdio.h>
#include <sys/types.h>

#include <qobject.h>
#include <qstring.h>
#include <qsocketnotifier.h>
#include <qtimer.h>

#include "mplayerfile.h"
#include "sighandler.h"

namespace emotion
{
	class Mixer;

	/** Controls the mplayer application */
	class Mplayer
		: public QObject,
		SignalHandler
	{
			Q_OBJECT
		public:
			Mplayer ();
			virtual ~Mplayer ();

			/** Start an instance of mplayer playing the specified file */
			virtual void play (const MplayerFile);

			/** Returns true if an instance of mplayer is running */
			virtual bool isRunning () const;

			/** Returns true if subtitles are being displayed */
			virtual bool getSubs() const;

			/** handle the given signal (SIGPIPE, SIGCHLD) */
			virtual void handleSignal (int sig);

			enum {
				QUIT_NULL,
				QUIT_USER,
				QUIT_EOF
			};

		public slots:
			virtual void seekTo(int percent);
			/** Seek forward (ignored if not running) */
			virtual void forward ();
			/** Pause playback (ignored if not running) */
			virtual void pause ();
			/** Quit mplayer (ignored if not running). As of now, this quits
			 * quickly but uses a method that seems to have side effects. Those
			 * side effects don't break anything on my system... */
			virtual void quit ();
			/** Seek back (ignored if not running) */
			virtual void back ();

			/** Toggle fullscreen playback on/off (ignored if not running) */
			virtual void toggleFullscreen ();
			/** Toggle subtitle display on/off (ignored if not running) */
			virtual void toggleSubtitles();

		signals:
			/** Emitted when mplayer is launched */
			virtual void mplayerStarted ();

			/** Emitted when mplayer stops on its own (i.e. end of file). This
			 * signal is NOT emitted by quit() */
			virtual void mplayerEOF ();
			/** Emitted when mplayer is quit by the user, either through
			 * pressing Q in mplayer */
			virtual void mplayerQuit ();
			/** Emitted when mplayer is quit by eMotion, by calling quit() */
			virtual void mplayerKilled ();

			/** Called when mplayer outputs some stuff */
			virtual void mplayerOutput (const char *);

			/** Called periodically with the percentage position in the file as
			 * a parameter */
			virtual void mplayerPercent(int p);

		private slots:
			/** Check to see if mplayer is still running, and poll it for
			 * position */
			virtual void pollMplayer ();
			/** Cleanup after mplayer exited */
			virtual void cleanup ();
			/** Called when mplayer has written some data to standard out */
			virtual void dataReceived (int);

		private:
			enum state_t
			{
				RUNNING,
				NOTRUNNING
			};

			/** pipes with which to communicate with the mplayer child process
			 * */
			int _fd_send[2];
			/** pipes with which to communicate with the mplayer child process
			 * */
			int _fd_recv[2];

			int _paused;
			int _fullscreen;
			int _subtitles;
			int _quitstate;

			/** pid of running mplayer process, or undefined otherwise */
			pid_t _id;
			state_t _state;
			/** timer used for polling mplayer to see if its still running */
			QTimer _timer;

			/** the last line returned from mplayer's output */
			QString _buffer;

			/** this object calls our dataReceived(int) slot when mplayer has
			 * written data to standard out (or is null if mplayer is not
			 * running) */
			QSocketNotifier *_notifier;

			/** Send a single text command (slave mode command) to mplayer. A
			 * newline is added to the provided command. This is ignored if
			 * mplayer is not running */
			void sendCommand (const QString &);
			/** Send mplayer the quit command */
			void sendQuit ();
			/** Send mplayer SIGTERM */
			void sendTerm ();
			/** Send mplayer SIGKILL */
			void sendKill ();

			/** Returns true if mplayer process still is running */
			bool pingChild();
	};
}

#endif
