/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: playlist.cpp,v 1.49 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <cassert>
#include <algorithm>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <qframe.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "errormediator.h"
#include "globalconfiguration.h"
#include "icons.h"
#include "playlist.h"

using namespace emotion;

Playlist::Playlist (QMainWindow *mainwin, QWidget *parent, const char *name, WFlags f)
	: QFrame (parent, name, f),
	_vbox (this),
	_vlayout (this),
	_list (0, 1, &_vbox),
	_buttonbar ("Playlist Operations", mainwin, &_vbox),
	_addfile (&_buttonbar, IconSet(emotionIcons::add_1, emotionIcons::add_2,
			emotionIcons::add_3)),
	_addfilemenu (&_addfile),
	_rmfile (&_buttonbar, IconSet(emotionIcons::remove_1, emotionIcons::remove_2,
			emotionIcons::remove_3)),
	_rmfilemenu (&_rmfile),
	_spacer (&_buttonbar),
	_loop (&_buttonbar, IconSet(IconSet(emotionIcons::unloop_1,
					emotionIcons::unloop_2, emotionIcons::unloop_3),
				IconSet(emotionIcons::loop_1, emotionIcons::loop_2,
					emotionIcons::loop_3))),
	_up (&_buttonbar, IconSet(emotionIcons::playlistup_1, emotionIcons::playlistup_2,
			emotionIcons::playlistup_3)),
	_down (&_buttonbar, IconSet(emotionIcons::playlistdown_1,
			emotionIcons::playlistdown_2, emotionIcons::playlistdown_3))
{
	//cout << "Playlist()" << endl;
	setFrameStyle(QFrame::NoFrame);

	QToolTip::add(&_addfile, "[Control-A] Add File(s)");
	QWhatsThis::add(&_addfile, "Click this to add files to the playlist. "
			"Alternatively, click and hold for more options");
	QToolTip::add(&_rmfile, "[Control-Shift-A] Remove Selected Files");
	QWhatsThis::add(&_rmfile, "Click this to remove the current selection "
			" from the playlist. Alternatively, click and hold for "
			"more options");
	QToolTip::add(&_loop, "[Control-L] Toggle playlist loop");
	QWhatsThis::add(&_loop, "Enable or disable loop playback. If enabled, "
			"eMotion will restart at the beginning of the playlist if "
			"the end is reached.");
	QToolTip::add(&_up, "[Control-Up] Move selected files up");
	QWhatsThis::add(&_up, "Move the currently selected files up "
			"in the playlist");
	QToolTip::add(&_down, "[Control-Down] Move selected files down");
	QWhatsThis::add(&_down, "Move the currently selected files down "
			"in the playlist");

	QMimeSourceFactory::defaultFactory()->setPixmap("active",
			QPixmap(emotionIcons::active));
	QWhatsThis::add(&_list, "The playlist shows what files will be played. The "
			"<img src=\"active\"> (active file) icon indicates the current "
			"file. This is the file that will be played if you press the play "
			"button. You may also double-click on any file listed to play it.");

	_up.setAutoRepeat(TRUE);
	_down.setAutoRepeat(TRUE);

	_loop.setToggleButton(TRUE);

	_addfile.setPopup (&_addfilemenu);
	_addfile.setPopupDelay (300);
	_rmfile.setPopup (&_rmfilemenu);
	_rmfile.setPopupDelay (300);

	_vlayout.setAutoAdd (TRUE);

	_buttonbar.setStretchableWidget (&_spacer);

	_buttonbar.setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
				QSizePolicy::Maximum));

	_list.setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
				QSizePolicy::MinimumExpanding));
	_list.setShowGrid (FALSE);
	_list.setRowMovingEnabled (FALSE);
	_list.setColumnMovingEnabled (FALSE);
	_list.setReadOnly(TRUE);
	_list.setSorting(FALSE);
	_list.verticalHeader()->hide();
	_list.setLeftMargin(0);
	_list.horizontalHeader()->hide();
	_list.setTopMargin(0);
	_list.setMinimumSize(QSize(72, 16));

	_list.horizontalHeader()->setLabel(0, "File");
	_list.setColumnStretchable (0, TRUE);
	_list.setSelectionMode (QTable::Multi);

	QObject::connect(&_list, SIGNAL(doubleClicked(int,int,int,const QPoint&)),
			this, SLOT(listItemClicked(int,int,int,const QPoint&)));
}

Playlist::~Playlist ()
{
}

void Playlist::fileAdded(const vector<QString> &files, int pos)
{
	int size = files.size();

	_list.setNumRows(_list.numRows() + size);

	for (vector<QString>::const_iterator it=files.begin();
			it != files.end();
			it++, pos++)
	{
		_list.setItem(pos, 0, new QTableItem(&_list, QTableItem::Never, *it));
		_list.setPixmap(pos, 0, QPixmap(emotionIcons::blank));
	}
}

void Playlist::fileRemoved(const vector<int> &positions)
{
	for (vector<int>::const_iterator it=positions.end() - 1;
			it >= positions.begin();
			it--)
		_list.removeRow(*it);
	_list.clearSelection();
}

ImageButton *Playlist::getRemove()
{
	return &_rmfile;
}

ImageButton *Playlist::getAdd()
{
	return &_addfile;
}

vector<int> Playlist::getSelection() const
{
	vector<int> rows;

	for (int i=0; i<_list.numSelections(); i++)
	{
		QTableSelection sel(_list.selection(i));

		if (sel.topRow() < 0 || sel.bottomRow() < 0)
			continue;

		for (int j=sel.topRow(); j<=sel.bottomRow(); j++)
			rows.push_back(j);
	}

	if (!rows.empty())
		sort(rows.begin(), rows.end());

	return rows;
}

ImageButton *Playlist::getUp()
{
	return &_up;
}

ImageButton *Playlist::getDown()
{
	return &_down;
}

void Playlist::setSelection(const vector<int> &rows)
{
	if (rows.empty())
		return;

	_list.clearSelection(false);

	QTableSelection sel;
	sel.init(rows[0], 0);

	int current;
	int prev;

	current = rows[0];
	prev = rows[0] - 1;

	for (vector<int>::const_iterator it=rows.begin();
			it != rows.end();
			it++)
	{
		current = *it;

		if (current != prev + 1)
		{
			sel.expandTo(prev, 0);
			_list.addSelection(sel);
			
			sel.init(current, 0);
		}

		prev = current;
	}

	sel.expandTo(current, 0);
	_list.addSelection(sel);
}

void Playlist::fileSwapped (int i, int j)
{
	_list.swapRows(i, j);
	_list.updateCell(i, 0);
	_list.updateCell(j, 0);
}

QPopupMenu *Playlist::getAddMenu()
{
	return &_addfilemenu;
}

void Playlist::fileDeselected(int i)
{
	_list.setPixmap(i, 0, QPixmap(emotionIcons::blank));
}

void Playlist::fileSelected(int i)
{
	_list.setPixmap(i, 0, QPixmap(emotionIcons::active));
}

void Playlist::listItemClicked(int row, int col, int, const QPoint &)
{
	// select the item clicked on
	vector<int> sel(1);
	sel[0] = row;
	setSelection(sel);

	// and pass the signal up
	emit fileClicked(row);
}

ImageButton *Playlist::getLoop()
{
	return &_loop;
}

QPopupMenu *Playlist::getRemoveMenu()
{
	return &_rmfilemenu;
}
