/*
 * The author disclaims copyright to the included source code. It is placed in
 * the public domain. In place of a lengthy legal notice, here is a blessing:
 *
 * 	May you do good and not evil.
 * 	May you find forgiveness for yourself and forgive others.
 * 	May you share freely, never taking more than you give.
 *
 * $Id: playlistmodel.cpp,v 1.9 2003/10/05 00:09:04 pyromage Exp $
 *
 */

#include <fstream>

#include "errormediator.h"
#include "playlistmodel.h"

using namespace emotion;

PlaylistModel::PlaylistModel()
	: _position(-1)
{
}

PlaylistModel::~PlaylistModel()
{
}

void PlaylistModel::add(const vector <MplayerFile> &files)
{
	if (files.empty())
		return;

	int pos = _list.size();

	vector <QString> added;

	for (const_iterator it=files.begin();
			it != files.end();
			it++)
	{
		_list.push_back(*it);
		added.push_back(it->getName());
	}

	emit fileAdded(added, pos);

	if (_list.size() == files.size())
		setPosition(0);
}

void PlaylistModel::remove (const vector<int> &positions)
{
	if (positions.empty())
		return;

	bool newpos = false;

	for (vector<int>::const_iterator it=positions.end() - 1;
			it >= positions.begin();
			it--)
	{
		_list.erase(_list.begin() + *it, _list.begin() + *it + 1);

		if (_position > *it)
			_position--;
		else if (_position == *it)
			newpos = true;
	}

	emit fileRemoved (positions);

	if (newpos)
		emit fileSelected(_position);
}

int PlaylistModel::size() const
{
	return _list.size();
}

void PlaylistModel::swap(int i, int j)
{
	MplayerFile tmp(_list[i]);
	_list[i] = _list[j];
	_list[j] = tmp;

	if (_position == i)
		_position = j;
	else if (_position == j)
		_position = i;

	emit fileSwapped(i, j);
}

void PlaylistModel::advance()
{
	// do nothing if list is empty
	if (_list.empty())
		return;

	if (_position < 0)
		_position = 0;
	else if (_position >= _list.size())
		_position = 0;
	else
	{
		emit fileDeselected(_position);
		_position++;
	}

	if (_position < _list.size() && _position >= 0)
		emit fileSelected(_position);
}

void PlaylistModel::retreat()
{
	if (_list.empty())
		return;

	if (_position < 0)
		_position = _list.size() - 1;
	else if (_position >= _list.size())
		_position = _list.size() - 1;
	else
	{
		emit fileDeselected(_position);
		_position--;
	}

	if (_position < _list.size() && _position >= 0)
		emit fileSelected(_position);
}

MplayerFile *PlaylistModel::get(int i)
{
	if (_list.empty())
		return NULL;
	if (i < 0 || i >= _list.size())
		return NULL;

	return &_list[i];
}

int PlaylistModel::getPosition() const
{
	if (_position >= _list.size())
		return -1;
	else if (_position < 0)
		return -1;

	return _position;
}

void PlaylistModel::setPosition(int i)
{
	if (_position >= 0 && _position < _list.size())
		emit fileDeselected(_position);

	_position = i;

	if (_position >= 0 && _position < _list.size())
		emit fileSelected(_position);
}

void PlaylistModel::clear()
{
	vector<int> positions(_list.size());
	for (int i=0; i<positions.size(); i++)
		positions[i] = i;
	remove(positions);
}

bool PlaylistModel::empty() const
{
	return _list.empty();
}

void PlaylistModel::write(const QString &file) const
{
	ofstream out (file.ascii());

	if (!out)
	{
		ErrorMediator::instance()->submitError(Exception(
					QString("Could not open file for writing:") + file));
		return;
	}
	
	for (vector<MplayerFile>::const_iterator it=_list.begin();
			it != _list.end();
			it++)
		out << (*it).getPath() << endl;

	out.close();
}

void PlaylistModel::read(const QString &playlist)
{
	ifstream in (playlist.ascii());
	vector<MplayerFile> files;

	if (!in)
	{
		ErrorMediator::instance()->submitError(Exception(QString( 
						"Could open file:") + playlist));
		return;
	}
	
	clear();

	while (!in.eof())
	{
		char buf[4096];
		in.getline(buf, 4096);
		QString filename(buf);

		filename.stripWhiteSpace();

		// skip comments
		if (filename.startsWith("#"))
			continue;
		else if (filename.length() == 0)
			continue;

		files.push_back(MplayerFile(filename, PlayOptions()));
	}

	in.close();

	add(files);
}
